@@ -346,7 +346,7 @@ journal7 = nulljournal {jtxns =
              tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/01",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="*",
              tdescription="opening balance",
              tcomment="",
@@ -362,7 +362,7 @@ journal7 = nulljournal {jtxns =
              tsourcepos=nullsourcepos,
              tdate=parsedate "2007/02/01",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="*",
              tdescription="ayres suites",
              tcomment="",
@@ -378,7 +378,7 @@ journal7 = nulljournal {jtxns =
              tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/02",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="*",
              tdescription="auto transfer to savings",
              tcomment="",
@@ -394,7 +394,7 @@ journal7 = nulljournal {jtxns =
              tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/03",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="*",
              tdescription="poquito mas",
              tcomment="",
@@ -410,7 +410,7 @@ journal7 = nulljournal {jtxns =
              tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/03",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="*",
              tdescription="verizon",
              tcomment="",
@@ -426,7 +426,7 @@ journal7 = nulljournal {jtxns =
              tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/03",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="*",
              tdescription="discover",
              tcomment="",
