@@ -13,6 +13,8 @@ where
 import Data.List
 import Data.Maybe
 import Data.Ord
+import Data.HashSet (size, fromList)
+import Data.Text (pack)
 import Data.Time.Calendar
 import System.Console.CmdArgs.Explicit
 import Text.Printf
@@ -21,13 +23,15 @@ import qualified Data.Map as Map
 import Hledger
 import Hledger.Cli.Options
 import Prelude hiding (putStr)
-import Hledger.Utils.UTF8IOCompat (putStr)
+import Hledger.Cli.Utils (writeOutput)
 
 
 statsmode = (defCommandMode $ ["stats"] ++ aliases) {
   modeHelp = "show some journal statistics" `withAliases` aliases
  ,modeGroupFlags = Group {
-     groupUnnamed = []
+     groupUnnamed = [
+        flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE[.FMT]" "write output to FILE instead of stdout. A recognised FMT suffix influences the format."
+        ]
     ,groupHidden = []
     ,groupNamed = [generalflagsgroup1]
     }
@@ -37,7 +41,7 @@ statsmode = (defCommandMode $ ["stats"] ++ aliases) {
 -- like Register.summarisePostings
 -- | Print various statistics for the journal.
 stats :: CliOpts -> Journal -> IO ()
-stats CliOpts{reportopts_=reportopts_} j = do
+stats opts@CliOpts{reportopts_=reportopts_} j = do
   d <- getCurrentDay
   let q = queryFromOpts d reportopts_
       l = ledgerFromJournal q j
@@ -45,7 +49,7 @@ stats CliOpts{reportopts_=reportopts_} j = do
       intervalspans = splitSpan (intervalFromOpts reportopts_) reportspan
       showstats = showLedgerStats l d
       s = intercalate "\n" $ map showstats intervalspans
-  putStr s
+  writeOutput opts s
 
 showLedgerStats :: Ledger -> Day -> DateSpan -> String
 showLedgerStats l today span =
@@ -63,7 +67,7 @@ showLedgerStats l today span =
         ,("Transactions", printf "%d (%0.1f per day)" tnum txnrate)
         ,("Transactions last 30 days", printf "%d (%0.1f per day)" tnum30 txnrate30)
         ,("Transactions last 7 days", printf "%d (%0.1f per day)" tnum7 txnrate7)
-        ,("Payees/descriptions", show $ length $ nub $ map tdescription ts)
+        ,("Payees/descriptions", show $ size $ fromList $ map (pack . tdescription) ts)
         ,("Accounts", printf "%d (depth %d)" acctnum acctdepth)
         ,("Commodities", printf "%s (%s)" (show $ length cs) (intercalate ", " cs))
       -- Transactions this month     : %(monthtxns)s (last month in the same period: %(lastmonthtxns)s)
